/*
 * An opener for regions of a memory card.
 *
 * Copyright (C) 2021, 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <resource/resource_server.h>

#include "msc_region_accessor.h"
#include "msc_region_opener.h"



MscRegionOpener::~MscRegionOpener()
{
}

bool MscRegionOpener::accessing_directory(flags_t flags, fileid_t fileid)
{
    (void) flags; (void) fileid;
    return false;
}

bool MscRegionOpener::accessing_file(flags_t flags, fileid_t fileid)
{
    (void) flags; (void) fileid;
    return true;
}

/* Test if a directory is empty. */

bool MscRegionOpener::directory_is_empty(fileid_t fileid)
{
    (void) fileid;
    return false;
}

/* Return a file identifier for the given 'path'. */

long MscRegionOpener::get_fileid(const char *path, flags_t flags, fileid_t *fileid)
{
    (void) flags;

    // Convert the path to an integer for use as the file identifier.

    if (!(strlen(path) && isdigit(path[0])))
        return -L4_ENOENT;

    *fileid = atoi(path);
    return L4_EOK;
}

/* Return a new accessor for 'fileid'. */

long MscRegionOpener::make_accessor(flags_t flags, fileid_t fileid,
                                    Accessor **accessor)
{
    // NOTE: Not testing for create or write flags.

    (void) flags;

    // Find the given partition and initialise the accessor with the appropriate
    // details.

    uint32_t start_block, num_blocks;

    if (_ops->get_partition(fileid, &start_block, &num_blocks))
        return -L4_ENOENT;

    *accessor = new MscRegionAccessor(_ops, start_block, num_blocks, fileid);
    return L4_EOK;
}

/* Return a directory accessor for 'fileid'.
   NOTE: Currently unimplemented. */

long MscRegionOpener::make_directory_accessor(flags_t flags, fileid_t fileid,
                                              DirectoryAccessor **accessor)
{
    (void) flags; (void) fileid; (void) accessor;
    return -L4_EPERM;
}



/* Make a new directory. */

long MscRegionOpener::make_directory_object(const char *path, sys_mode_t mode)
{
    (void) path; (void) mode;
    return -L4_EPERM;
}

/* Remove a filesystem object. */

long MscRegionOpener::remove_object(fileid_t fileid)
{
    (void) fileid;
    return -L4_EPERM;
}

/* Rename a filesystem object, placing source inside the parent of target. */

long MscRegionOpener::rename_object(const char *source, const char *target)
{
    (void) source; (void) target;
    return -L4_EPERM;
}

/* Populate a memory region with statistics metadata for a filesystem object.
   NOTE: Currently unimplemented. */

long MscRegionOpener::stat_object(const char *path, void *base, offset_t size)
{
    (void) path; (void) base; (void) size;
    return -L4_EPERM;
}

/* Unlink a filesystem object. */

long MscRegionOpener::unlink_object(fileid_t parent_fileid, fileid_t fileid)
{
    (void) parent_fileid; (void) fileid;
    return -L4_EPERM;
}

// vim: tabstop=4 expandtab shiftwidth=4
